#!/usr/bin/perl -w
#
# Copyright (C) 2003,2004 Robert van den Aker <robert2@dds.nl>
# This script uses functions that are copyright (C) 2002 Mitel Networks
# Corporation.
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307,
# USA.
#
#----------------------------------------------------------------------

package esmith::FormMagick::Panel::printers;

use strict;
use esmith::ConfigDB;
use esmith::DB::db;
use esmith::FormMagick;
use esmith::cgi;
use Exporter;

our @ISA = qw(esmith::FormMagick Exporter);

our @EXPORT = qw(
    showInitial printerSync
);

our $configdb = esmith::ConfigDB->open() ||
    die "Could not open configuration database";


=pod

=head1 NAME

esmith::FormMagick::Panel::printers - CUPS printers panel functions

=head1 SYNOPSIS

    use esmith::FormMagick::Panel::printers;
    my $panel = esmith::FormMagick::Panel::printers->new();
    $panel->display();

=head1 DESCRIPTION


=head2 new

Exactly as for esmith::FormMagick.

=cut

sub new
{
    shift;
    my $self = esmith::FormMagick->new();
    $self->{calling_package} = (caller)[0];
    bless $self;
    return $self;
}


=pod

=head2 getCupsUrl

Returns the URL for the CUPS web interface.

=cut

sub getCupsUrl
{
    my $url = "http://". ($configdb->get('LocalIP')->value() ||
	'localhost').":631/";
    return ($url);
}


=pod

=head2 getSyncStatus

Compares CUPS printers configuration file and cups database to determine
whether the printer-sync event should be run.

=cut

sub getSyncStatus
{
    my $self = $_[0];
    my $cupsdb = esmith::DB::db->open_ro("/home/e-smith/cups/db") ||
	die "Could not open cups database";
    my $printersconf = "/etc/cups/printers.conf";
    my $ppddir = "/etc/cups/ppd";
    my $lastsync = $configdb->get('cups')->prop('LastSync') || "0";
    my $confmodtime = (stat($printersconf))[9] ||
	return "<FONT COLOR=\"Red\">Cannot stat $printersconf!</FONT>";
    my $ppddirmodtime = (stat($ppddir))[9] || "0";
    my @confprinters;
    open (PRINTERSCONF, "<$printersconf");
    while (<PRINTERSCONF>) {
	chomp;
	/^<(Default)?Printer ([A-Za-z1-9][A-Za-z0-9\-\_]*)>$/ &&
	push (@confprinters, $2);
    }
    close (PRINTERSCONF);
    my $confprinters = join("",sort(@confprinters));
    my @dbprinters;
    foreach ($cupsdb->get_all_by_prop(type => 'printer')) {
	push (@dbprinters, $_->key);
    }
    my $dbprinters = join("",sort(@dbprinters));
    unless (-f "$printersconf.O") {
	return $self->localise('NO_PRINTERS_TO_SYNC');
    }
    elsif ($lastsync < $confmodtime &&
    ($confprinters ne $dbprinters || $lastsync < $ppddirmodtime)) {
	return $self->localise('PRINTERS_OUT_OF_SYNC');
    }
    else {
	return $self->localise('PRINTERS_IN_SYNC');
    }
}


=pod

=head2 showInitial

Displays the printers panel.

=cut

sub showInitial
{
    my $self = shift;
    my $q = $self->{cgi};

    print "<tr><td>";
    print $q->p("<a href=\"". &getCupsUrl."\">".
    $self->localise('CLICK_HERE'). "</a> ".
    $self->localise('CUPS_DESCRIPTION_BEGIN')." ". &getCupsUrl.
    " ". $self->localise('CUPS_DESCRIPTION_END'));
    print $q->p ($q->b ($self->localise('SYNC_DESC_HEADER')));
    print $q->p ($self->localise('SYNC_DESCRIPTION'));
    print $q->table ({border => 0, cellspacing => 0, cellpadding => 4},
    $q->Tr (esmith::cgi::genCell ($q, &getSyncStatus($self)),
	esmith::cgi::genCell ($q, $q->b ($q->submit (
	-value => $self->localise('SYNCHRONIZE'))))));
    print "</td></tr>";
    return '';
}


=pod

=head2 printerSync

Signals the printer-sync event.

=cut

sub printerSync
{
    my $self = shift;
    my $q = $self->{cgi};

    system ("/sbin/e-smith/signal-event", "printer-sync") == 0
	or return $self->error('PRINTER_SYNC_ERROR', 'Initial');

    return $self->success('PRINTER_SYNC_SUCCESS', 'Initial');
}


1;
